<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

add_action('admin_init', function() {
	gartentechnik_com_debug('admin_init_start');
	wp_enqueue_style('wp-color-picker');
	wp_register_script(
		'gartentechnik_com_admin_scripts',
		plugins_url('resources/js/admin_dummy.js', __FILE__),
		array(
			# Abhaengigkeitendie in den GT-JS, die via API kommen, benutzt werden
			'jquery', 'wp-color-picker'
		),
		null,
		false
	);
	wp_enqueue_script('gartentechnik_com_admin_scripts');
	wp_localize_script(
		'gartentechnik_com_admin_scripts',
		'gartentechnik_com_js_translation',
		array('url' => admin_url( 'admin-ajax.php' ))
	);

	require_once(__DIR__ . '/_gtcom_adminbereich_optionen.php');
	gartentechnik_com_debug('admin_init_end');
}, 0, 0);

add_filter('custom_menu_order', '__return_true');
add_filter('menu_order', function($order = []) use ($rolle_des_aktuellen_users) {
	gartentechnik_com_debug('menu_order_start');
	global $submenu;

	$admin_slug = 'gartentechnik_com-admin';
	if($rolle_des_aktuellen_users == 'hersteller') {
		$admin_slug .= '-herstelleradmin';
	}

	if(!$submenu || !array_key_exists($admin_slug, $submenu) || !$submenu[$admin_slug]) return $order;# Probleme in Netzwerkadmin-Kontext unterbunden

	$liste = [];
	foreach($submenu[$admin_slug] as &$eintrag) {
		$array = explode('-', $eintrag[0], 2);
		$eintrag[0] = array_key_exists(1, $array) ? $array[1] : null;
		if(!preg_match('/^[0-9]+$/', $array[0])) {
			$array[0] = 0;
		}
		$liste[] = [
			'eintrag'	=> $eintrag,
			'order'		=> ($array[0] * 1),
		];
	}
	usort($liste, function($a, $b) {
		return $a['order'] - $b['order'];
	});
	$submenu[$admin_slug] = [];
	foreach($liste as $e) {
		$submenu[$admin_slug][] = $e['eintrag'];
	}
	gartentechnik_com_debug('menu_order_end');
	return $order;
}, 1, 0);

$gartentechnik_com_adminmenue_bar_hook = 'add_admin_bar_menus';
if(is_admin()) {
	$gartentechnik_com_adminmenue_bar_hook = 'admin_menu'; // 'network_admin_menu'
} else if (is_network_admin()) {
	$gartentechnik_com_adminmenue_bar_hook = 'network_admin_menu';
}

add_action($gartentechnik_com_adminmenue_bar_hook, function() use(
	$app, $response, $rolle_des_aktuellen_users, $ist_admin_kontext, $ist_superadmin_kontext
) {
	gartentechnik_com_debug('adminmenue_bar_start');
	if(!$rolle_des_aktuellen_users) return;

	if(!$ist_admin_kontext) return;

	$menue = $response->admin_menue();

	$blog_info_slug = 'blog-info';
	$menue[] = [
		'slug'	=> $blog_info_slug,
		'titel'	=> '80-Blog-Übersicht',
		'erforderliche_rolle'	=> ['superadmin'],
	];

	$wp_api_key_slug = 'wp-api-key';
	$menue[] = [
		'slug'	=> $wp_api_key_slug,
		'titel'	=> '95-API-Key',
		'erforderliche_rolle'	=> ['superadmin','admin'],
	];

	$gefiltertes_menue = [];
	foreach($menue as $e) {
		foreach($e['erforderliche_rolle'] as $erforderliche_rolle) {
			if($erforderliche_rolle == $rolle_des_aktuellen_users) {
				$gefiltertes_menue[] = $e;
				break;
			}
		}
	}
	$counter = 0;
	foreach($gefiltertes_menue as $eintrag) {
		if($eintrag['slug']) $counter++;
	}
	if($counter == 0) return;

	$adminbereich_slug = 'gartentechnik_com-admin';
	if($rolle_des_aktuellen_users == 'hersteller') {
		$adminbereich_slug .= '-herstelleradmin';
	}
	$adinbereich_topmenue_id = 'gartentechnik_com-topmenue';
	$title_prefix = 'Gartentechnik.com Verwaltung: ';

	if(is_admin()) {
		add_menu_page(
			$title_prefix . $menue[0]['titel'],
			'Gartentechnik',
			'read',
			(\GartentechnikCom\Register::api_key_ist_ungueltig() ? esc_url(admin_url('admin.php')) . '?page=' . $adminbereich_slug . '-' . $wp_api_key_slug : $adminbereich_slug),
			($rolle_des_aktuellen_users == 'hersteller' ? function() use($adminbereich_slug) {
				header("HTTP/1.1 301 MovedPermanently");
				header('Location: ' . esc_url(admin_url('admin.php')) . '?page=' . $adminbereich_slug . '-allgemein');
			} : ''),
			'dashicons-feedback',
			2 # https://developer.wordpress.org/resource/dashicons/
		);
	}

	$top_menue = \GartentechnikCom\Register::wp_admin_topmenue();
	$hauptmenuepunkt_url = esc_url(admin_url('admin.php')) . '?page=' . $adminbereich_slug;
	if(\GartentechnikCom\Register::api_key_ist_ungueltig()) {
		$hauptmenuepunkt_url = esc_url(admin_url('admin.php')) . '?page=' . $adminbereich_slug . '-' . $wp_api_key_slug;
	}
	array_push($top_menue, array(
		'id'	=> $adinbereich_topmenue_id,
		'title'	=> 'root-<span class="gartentechnik_com_admin_top_bar"><span class="icon"></span><span class="text">Gartentechnik</span>',
		'href'	=> $hauptmenuepunkt_url,
	));

	foreach($gefiltertes_menue as $eintrag) {
		$slug = $adminbereich_slug . "-" . $eintrag['slug'];
		if(!$eintrag['slug']) $slug = $adminbereich_slug;

		$func = function() use($response) {
			if($response->is_404()) {
				gartentechnik_com_debug('404');
				header("HTTP/1.1 404 NotFound");
				exit(0);
			} else if($response->redirect_location()) {
				gartentechnik_com_debug('301');
				header("HTTP/1.1 301 MovedPermanently");
				header('Location: ' . $response->redirect_location());
				exit(0);
			} else if($response->content_type() != 'text/html') {
				gartentechnik_com_debug('plain content');
				header("HTTP/1.1 200 OK");
				header('Content-Type: ' . $response->content_type() . "; charset=" . $response->charset());
				if($response->content_disposition()) {
					header("Content-Disposition: " . $response->content_disposition());
				}
				echo $response->main_content();
				exit(0);
			}
			$header = '';
			if($response->html_head_titel()) {
				$header = "<h1>" . $response->html_head_titel() . "</h1>";
			}
			echo
				'<div class="wrap gartentechnik_com-admin">'
				. '<div id="loadingStatus"><div id="loader-wrapper"><div id="loader"></div></div></div>'
				. $header
				. $response->main_content();
			include (__DIR__ . '/templates/footer_kontaktbox.html');
			echo
				"</div>";
		};
		if($eintrag['slug'] == $blog_info_slug) {
			$func = function() use(
				$app, $rolle_des_aktuellen_users, $ist_admin_kontext, $ist_superadmin_kontext
			) {
				require_once(__DIR__ . '/templates/blog_uebersicht.tpl.php');
			};
		} elseif($eintrag['slug'] == $wp_api_key_slug) {
			$func = function() {
				require_once(__DIR__ . '/templates/api-key.tpl.php');
			};
		}

		$titel = $eintrag['titel'];
		$array = explode('-', $eintrag['titel'], 2);
		if(count($array) == 2) {
			$titel = $array[1];
		}
		if(is_admin()) {
			$hook_name = add_submenu_page(
				$adminbereich_slug,
				$title_prefix . $titel,
				$eintrag['titel'],
				'read',
				$slug,
				$func
			);
		}
		$slug_url = esc_url(admin_url('admin.php')) . '?page=' . $slug;
		array_push($top_menue, array(
			'id' => $adinbereich_topmenue_id . $slug,
			'title' => $eintrag['titel'],
			'parent' => $adinbereich_topmenue_id,
			'href'	=> $slug_url,
		));
	}
	\GartentechnikCom\Register::wp_admin_topmenue($top_menue);
	gartentechnik_com_debug('adminmenu_bar_end');
}, 0, 0);

add_action('admin_bar_menu', function($wp_admin_bar) {
	gartentechnik_com_debug('admin_bar_menu_start');
	$list = \GartentechnikCom\Register::wp_admin_topmenue();
	$sortierbare_liste = [];
	foreach($list as $e) {
		$array = explode('-', $e['title'], 2);
		if($array[0] == 'root') $array[0] = -1;

		$e['title'] = $array[1];
		array_push($sortierbare_liste, array(
			'order'	=> $array[0],
			'entry'	=> $e,
		));
	}
	usort($sortierbare_liste, function($a, $b) {
		return $a['order'] - $b['order'];
	});
	foreach($sortierbare_liste as $e) {
		$wp_admin_bar->add_node($e['entry']);
	}
	gartentechnik_com_debug('admin_bar_menu_end');
}, 81, 1);

add_action('load-edit.php', function() {
	gartentechnik_com_debug('load_edit_start');
	$screen = get_current_screen();
	if($screen->id == 'edit-gartentechnik_com_me') {
		add_action('admin_footer', function() {
			include (__DIR__ . '/templates/footer_kontaktbox.html');
		});
	}
	gartentechnik_com_debug('load_edit_end');
});
?>
