<?php
namespace GartentechnikCom;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class MotoristPlugin {
	public static $response = null;
	private $domain = null;
	private $url_schema = null;
	private $current_path_prefix = null;
	private $is_currently_retrying_slug_generation = false;

	# VOID
	function __construct() {
	}
	# VOID
	public function init($url_schema, $domain, $current_path_prefix) {
		$this->domain = $domain;
		$this->url_schema = $url_schema;
		$this->current_path_prefix = $current_path_prefix;
	}
	# BOOLEAN
	public function ist_gtcom_request($request_method, $path) {
		return $this->_ist_gtcom_pfad($path);
	}
	# BOOLEAN
	private function _ist_gtcom_pfad($path) {
		$key_data = \GartentechnikCom\EinstellungsRegister::gib_gtcom_key_fuer_pfad($path);
		if(is_null($key_data)) {
			return false;
		}

		if(\GartentechnikCom\EinstellungsRegister::ist_gtcom_slug_aktiv($key_data['key'])) {
			return true;
		}

		return false;
	}
	# STRING
	private function _uebersetze_pfad($path) {
		$key_data = \GartentechnikCom\EinstellungsRegister::gib_gtcom_key_fuer_pfad($path);
		$path = '/' . $key_data['key'] . $key_data['suffix'];
		return $path;
	}
	# OBJ
	public function execute_gtcom_request(
		$request_method, $path, $cgi_parameter,
		$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
	) {
		$path = $this->_uebersetze_pfad($path);
		$api_response = $this->_call_api(
			true, $request_method, '/api' . $path, $cgi_parameter,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		self::$response = new Response();
		if(is_null($api_response) || !$api_response['is_json']) {
			if($api_response['response_code'] == '403') {
				self::$response->explicit_response_code('403 FORBIDDEN');
			} else {
				self::$response->is_404(true);
			}
			return self::$response;
		}

		$this->_fill_response(self::$response, $api_response['content'], true);
		return self::$response;
	}
	# OBJ
	public function execute_other(
		$cgi_parameter,
		$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
	) {
		if(!$ist_admin_kontext) $rolle_des_aktuellen_users = '';

		$api_response = $this->_call_api(
			true, 'GET', '/api/', $cgi_parameter,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		self::$response = new Response();
		if(is_null($api_response) || !$api_response['is_json']) {
			self::$response->is_404(true);
			return self::$response;
		}

		$this->_fill_response(self::$response, $api_response['content'], false);
		return self::$response;
	}
	# ARRAY
	public function gib_suchergebnisse($suche) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';
		$api_response = $this->_call_api(
			false, 'GET', '/json/suche', array('s' => $suche),
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		if(is_null($api_response) || !$api_response['is_json'] || is_null($api_response['content'])) {
			return array();
		}
		return $api_response['content'];
	}
	# ARRAY
	public function sende_anfrage($param) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';

		$path = '/json/anfrage';
		if(@$param['ist_adminanfrage']) {
			$path = '/json/adminbereich_anfrage';
		}
		$result = $this->_call_api(
			false, 'POST', $path, $param,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		if(is_null($result) || !$result['is_json']) {
			return array(
				'error' => '1',
				"antwort" => array(
					"versendet" => '0',
				),
			);
		}
		return $result['content'];
	}
	# ARRAY
	public function sende_produktrezension($param) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';
		$result = $this->_call_api(
			false, 'POST', '/json/produkt-rezension/', $param,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		if(is_null($result) || !$result['is_json']) {
			return array(
				'error' => '1',
				"antwort" => array(
					"versendet" => '0',
				),
			);
		}
		return $result['content'];
	}
	# ARRAY || NULL
	public function gib_widget_produktansicht($ids) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';
		gartentechnik_com_debug('produkt_widget_start');
		$result = $this->_call_api(
			false, 'POST', "/json/produktwidget", $ids,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		gartentechnik_com_debug('produkt_widget_end');
		if(is_null($result) || !$result['is_json']) return array();

		return $result['content'];
	}
	# STRING
	private function _lese_datei($file) {
		$fh = fopen($file, "r");
		if(!$fh) return '';

		$content = fread($fh, filesize($file));
		fclose($fh);

		return $content;
	}
	# BOOLEAN
	public function starte_api_key_testzeitraum(
		$domain,
		$ist_admin_kontext,
		$ist_superadmin_kontext,
		$rolle_des_aktuellen_users
	) {
		if($ist_superadmin_kontext) {
			\GartentechnikCom\Register::api_key_testzeitraum_fehlermeldung("Im Superadmin-Account kann ein Testzeitraum nicht gestartet werden.");
			return false;
		} elseif(!$ist_admin_kontext) {
			\GartentechnikCom\Register::api_key_testzeitraum_fehlermeldung("Nur als Admin der WP-Instanz darf ein Testzeitraum gestartet werden.");
			return false;
		}

		$result = $this->_call_api(
			false, 'POST', "/json/api_key_starte_testzeitraum", ['domain' => $domain],
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		if(is_null($result) || !$result['is_json'] || !$result['content']['erfolg']) {
			$msg = '';
			if($result && $result['is_json']) {
				$msg = $result['content']['meldung'];
			}
			if(!$msg) {
				$msg = "Fehler: Es konnte kein API-Key erstellt werden. Bitte versuchen sie es später noch einmal.";
			}
			\GartentechnikCom\Register::api_key_testzeitraum_fehlermeldung($msg);
			return false;
		}
		update_option('gartentechnik_com_einstellungen', ['gartentechnik_com_api_key' => $result['content']['generierter_api_key']], true);
		return true;
	}
	# ARRAY
	public function sende_admin_api_anfrage(
		$param, $files, $ist_admin_kontext,
		$ist_superadmin_kontext, $rolle_des_aktuellen_users,
		$request_content_type = null, $request_body = null
	) {
		$method = $param['api_method'];
		$path = $param['api_path'];
		unset($param['api_method']);
		unset($param['api_path']);
		unset($param['']);

		$path = preg_replace('/[^a-z0-9\/_\-\.]/', '', $path);
		$sub_path = $ist_superadmin_kontext ? 'superadmin' : 'motorist_admin';
		gartentechnik_com_debug('admin_api_start');

		if($files && array_key_exists('file', $files) && is_array($files['file'])) {
			gartentechnik_com_debug('upload_file');
			$param['file.name'] = $files['file']['name'];
			$param['file.mime_type'] = $files['file']['type'];
			if(!$files['file']['error']) {
				$content = $this->_lese_datei($files['file']['tmp_name']);
			}
			$param['file.content_base64'] = base64_encode($content);
		}

		$api_version = \GartentechnikCom\Register::gib_api_version();

		$api_path = "/json/$sub_path/$path";
		if(array_key_exists('v', $param) && preg_match('/^\d+$/', $param['v'])) {
			$api_version = $param['v'];
			if($api_version >= 3) {
				$api_path = "/$path";
			}
		}
		$result = $this->_call_api(
			false, $method, $api_path, $param,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users,
			$api_version,
			$request_content_type,
			$request_body
		);

		gartentechnik_com_debug('admin_api_end');
		return $result;
	}
	# STRING
	private function _gib_client_ip() {
		$client_ip = $this->_gib_client_ip_intern($_SERVER['SERVER_ADDR']);
		if(!$client_ip) {
			gartentechnik_com_debug("no client ip found");
			foreach(getenv() as $k => $v) {
				if(preg_match('/^\d+\.\d+\.\d+\.\d+[ ,]/', "$v ")) {
					gartentechnik_com_debug("E $k: $v");
				}
			}
			foreach($_SERVER as $k => $v) {
				if(preg_match('/^\d+\.\d+\.\d+\.\d+[ ,]/', "$v ")) {
					gartentechnik_com_debug("S $k: $v");
				}
			}
			return 'keine_wp_client_ip';
		}
		return $client_ip;
	}
	# STRING
	private function _gib_client_ip_intern($server_ip) {
		foreach([
			'HTTP_X_FORWARDED_FOR', 'HTTP_X_REAL_IP', 'X_REAL_IP', 'HTTP_CLIENT_IP',
			'HTTP_X_FORWARDED', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'HTTP_CF_CONNECTING_IP',
			'REMOTE_ADDR'
		] as $key) {
			$ip = getenv($key, true) ?: getenv($key);
			if($ip && $server_ip != $ip) {
				gartentechnik_com_debug("clientIP found in $key");
				return $ip;
			}
			$ip = @$_SERVER[$key];
			if($ip && $server_ip != $ip) {
				gartentechnik_com_debug("clientIP found in $key");
				return $ip;
			}
		}
		return '';
	}
	# HASH
	private function _korrigiere_parameter($cgi_parameter) {
		# Dots and spaces in variable names are converted to underscores. For example <input name="a.b" /> becomes $_REQUEST["a_b"].
		# http://it2.php.net/manual/it/language.variables.external.php
		foreach($cgi_parameter as $key => $value) {
			if(preg_match('/#/', $key)) {
				$neuer_key = str_replace('#', '.', $key);
				$cgi_parameter[$neuer_key] = $value;
				unset($cgi_parameter[$key]);
			}
		}
		return $cgi_parameter;
	}
	# STRING
	private function _gib_uri_parameter_string($request_param) {
		$request_param = $this->_korrigiere_parameter($request_param);
		$request_param_string = '?';
		foreach($request_param as $key => $val) {
			$request_param_string .= '&' . urlencode($key) . '=' . ($val);
		}
		if($request_param_string == '?') return '';

		return $request_param_string;
	}
	# HASH || NULL
	private function _call_api(
		$check_for_404, $request_method, $path, $cgi_parameter,
		$ist_admin_kontext, $ist_superadmin_kontext, $benutzerrolle,
		$api_version = null,
		$request_content_type = null,
		$request_body = null
	) {
		global $current_user;

		if(!$path) $path = '/';

		if(!$benutzerrolle) $benutzerrolle = 'user';

		if(
			($ist_admin_kontext || $ist_superadmin_kontext)
			&& @$cgi_parameter['page']
			&& preg_match(
				'/^gartentechnik_com\-admin\-?(.*)$/',
				sanitize_text_field(@$cgi_parameter['page']),
				$page_treffer
			)
		) {
			$sub_path = $page_treffer[1];
			if(!$sub_path) $sub_path = '';

			$path = '/api/motorist_admin/' . $sub_path;
			if($ist_superadmin_kontext) {
				$path = '/api/superadmin/' . $sub_path;
			}
			unset($cgi_parameter['page']);
		}

		$api_url = 'https://www.gartentechnik.com';
		$compress = true;
		$ist_externes_plugin = true;
		if(
			array_key_exists('GARTENTECHNIK_COM_IS_DOCKER_CONTEXT', $_SERVER)
			&& $_SERVER['GARTENTECHNIK_COM_IS_DOCKER_CONTEXT'] == '1'
		) {
			$compress = false;
			$api_url = 'http://app:8080';
			if($api_version && $api_version >= 3) {
				$api_url = 'http://concierge:80';
			}
			$ist_externes_plugin = false;
		}

		$response_ist_standardseite = false;
		if($request_method == 'GET' && $path == '/api/') {
			$response_ist_standardseite = true;
		}

		$api_timeout = 60;
		$timeout_ist_verkuerzt = false;
		if($response_ist_standardseite && $this->_anzahl_der_api_timeouts_der_letzten_5_minuten() >= 2) {
			gartentechnik_com_debug('api-timeout wird verkuerzt');
			$api_timeout = 2;
			$timeout_ist_verkuerzt = true;
		}

		# https://developer.wordpress.org/reference/classes/wp_http/request/
		if(!$api_version) {
			$api_version = \GartentechnikCom\Register::gib_api_version();
		}

		$api_key = \GartentechnikCom\EinstellungsRegister::gib_einstellung('gartentechnik_com_api_key');
		$api_key = preg_replace('/[^A-F0-9\-]+/', '', $api_key);
		if($api_version < 3) {
			$cgi_parameter['wordpress_api_key'] = $api_key;
		}

		gartentechnik_com_debug('gtcom_post_start');
		$wp_benutzerloginname = '';
		if($ist_admin_kontext && $current_user && $current_user->exists()) {
			$wp_benutzerloginname = $current_user->user_login . '(' . $current_user->user_email . ')';
			$api_timeout = 600;
		}
		$client_ip = $this->_gib_client_ip();
		$request = array(
			'method'				=> ($api_version >= 3 ? $request_method : 'POST'),
			'timeout'				=> $api_timeout,
			'httpversion'			=> '1.1',
			'compress'				=> $compress,
			'decompress'			=> true,
			'headers'				=> array(
				'X-Ist-Externes-Plugin'			=> ($ist_externes_plugin ? '1' : '0'),
				'X-Wordpress-Client-IP'			=> $client_ip,
				"X-Wordpress-Api-Key"			=> ($api_version >= 3 ? $api_key : '-'),
				"X-Wordpress-Host"				=> $this->domain,
				"X-Wordpress-Schema"			=> $this->url_schema,
				"X-Wordpress-PathPrefix"		=> $this->current_path_prefix,
				"X-Wordpress-Request-Method"	=> $request_method,
				"X-Wordpress-User-Agent"		=> getenv('HTTP_USER_AGENT'),
				"X-Wordpress-Ist-Admin-Kontext"	=> ($ist_admin_kontext ? '1' : '0'),
				"X-Wordpress-Benutzerrolle"		=> $benutzerrolle,
				"X-Wordpress-Plugin-Version"	=> \GartentechnikCom\Register::gib_plugin_version(),
				"X-Wordpress-Benutzerlogin"		=> $wp_benutzerloginname,
				"Accept-Encoding"				=> "gzip",
				"Content-Type"					=> ($request_content_type ? $request_content_type : "application/x-www-form-urlencoded"),
				"User-Agent"					=> "gt2-wp-plugin-v" . \GartentechnikCom\Register::gib_plugin_version(),
			),
			'sslverify'				=> true,
			'redirection'			=> 0,
			'blocking'				=> true,
			'reject_unsafe_urls'	=> false,
			'limit_response_size'	=> 0,
			'body'					=> ($request_content_type ? $request_body : $this->_korrigiere_parameter($cgi_parameter)),
		);

		$url_param = ($request_content_type ? $this->_gib_uri_parameter_string($cgi_parameter) : '');
		$response = wp_remote_post(
			$api_url . '/api/motorist_plugin/v' . $api_version . $path . $url_param,
			$request
		);
		$response_ist_fehlerhaft = is_wp_error($response);
		$response_header = ($response_ist_fehlerhaft ? [] : $response['headers']->getAll());
		$api_response_code = ($response_ist_fehlerhaft ? 0 : $response['response']['code']);
		gartentechnik_com_debug('api-response-code ' . $api_response_code);
		gartentechnik_com_debug('detail ' . @$response_header['x-response-detail']);
		gartentechnik_com_debug('IP+Agent ' . $client_ip . '+' . getenv('HTTP_USER_AGENT'));
		#foreach(getallheaders() as $k => $v) {
		#	gartentechnik_com_debug("H-$k: $v");
		#}
		$api_response = [
			'body'				=> $response_ist_fehlerhaft ? '' : $response['body'],
			'code'				=> $api_response_code,
			'ist_fehlerhaft'	=> $response_ist_fehlerhaft,
			'api_key_fehler'	=> $response_ist_fehlerhaft ? false : (array_key_exists('x-api-key-fehler', $response_header) ? true : false),
			'api_key_fehler_details'	=> $response_ist_fehlerhaft ? '' : (array_key_exists('x-api-key-fehler-details', $response_header) ? $response_header['x-api-key-fehler-details'] : ''),
			'api_key_ist_testaccount'	=> $response_ist_fehlerhaft ? false : (array_key_exists('x-api-key-ist-testaccount', $response_header) ? true : false),
			'api_key_testaccount_gueltigkeit_in_tagen'	=> $response_ist_fehlerhaft ? '0' : (array_key_exists('x-api-key-testaccount-gueltigkeit-in-tagen', $response_header) ? $response_header['x-api-key-testaccount-gueltigkeit-in-tagen'] : '0'),
			'api_key_darf_testaccount_beantragen'	=> $response_ist_fehlerhaft ? '0' : (array_key_exists('x-api-key-darf-testaccount-beantragen', $response_header) ? true : false),
		];
		$status_code = wp_remote_retrieve_response_code($response);
		unset($response);# RAM wieder freigeben... das Ding ist riesig
		gartentechnik_com_debug('gtcom_post_end');

		if($api_response['ist_fehlerhaft']) {
			gartentechnik_com_debug("API-Fehler");

			if(!$timeout_ist_verkuerzt) {
				$this->_speichere_api_timeoutfehler();
			}

			if($response_ist_standardseite) {
				$api_response = &$this->_lese_cache_fuer_standardseite();# Referenz... um Speicher zu sparen
				if(!$api_response) {
					return null;
				}
				gartentechnik_com_debug("Benutze Cache der Standardseite");
			} else {
				return null;
			}
		} else {
			if($response_ist_standardseite) {
				gartentechnik_com_debug('Schreibe Cache der Standardseite');
				$this->_schreibe_cache_fuer_standardseite($api_response);
			}
		}

		gartentechnik_com_debug('gtcom_post_json_start');
		if(!array_key_exists('content-type', $response_header)) {
			$response_header['content-type'] = '';
		}

		if (preg_match('/^application\/json(?:$|;)/', $response_header['content-type'])) {
			$result = array(
				"content" => json_decode($api_response['body'], true),
				"is_json" => true,
				"status_code" => $status_code,
				"response_code" => $api_response['code']
			);
		} else {
			$result = array(
				"content" => $api_response['body'],
				"is_json" => false,
				"status_code" => $status_code,
				"response_code" => $api_response['code']
			);
		}
		gartentechnik_com_debug('gtcom_post_json_end');
		if($check_for_404 && $result['is_json']) {
			$result['content']['ist_404'] = ($api_response['code'] == 404 ? true : false);
		}

		if($api_response['api_key_ist_testaccount']) {
			gartentechnik_com_debug('api-key-ist-testaccount');
			\GartentechnikCom\Register::api_key_ist_testaccount(true);
		}
		\GartentechnikCom\Register::api_key_testaccount_gueltigkeit_in_tagen($api_response['api_key_testaccount_gueltigkeit_in_tagen']);
		if($api_response['api_key_darf_testaccount_beantragen']) {
			\GartentechnikCom\Register::api_key_darf_testaccount_beantragen(true);
		}

		if($api_response['api_key_fehler']) {
			gartentechnik_com_debug('api-key-fehler');
			\GartentechnikCom\Register::api_key_ist_ungueltig(true);
			\GartentechnikCom\Register::api_key_fehler_details($api_response['api_key_fehler_details']);
			if($path == '/json/api_key_starte_testzeitraum') {
				return $result;
			}
			gartentechnik_com_debug('FEHLER: ' . $api_response['api_key_fehler_details']);
			return null;
		}
		return $result;
	}
	# INT
	private function _anzahl_der_api_timeouts_der_letzten_5_minuten() {
		$fehler = get_option('gartentechnik_com_api_timeoutfehler');
		if(!is_array($fehler)) return 0;

		$now = time();
		$count = 0;
		foreach($fehler as $zeitpunkt_des_fehlers) {
			if($zeitpunkt_des_fehlers + (5 * 60) > $now) {
				$count++;
			}
		}
		return $count;
	}
	# VOID
	private function _speichere_api_timeoutfehler() {
		gartentechnik_com_debug('speichere_timeoutfehler');
		$alte_fehler = get_option('gartentechnik_com_api_timeoutfehler');
		if(!is_array($alte_fehler)) $alte_fehler = [];

		$now = time();
		$neue_fehler = [$now];
		foreach($alte_fehler as $zeitpunkt_des_fehlers) {
			if($zeitpunkt_des_fehlers + (10 * 60) >= $now) {# nur 6 Minuten speichern
				array_push($neue_fehler, $zeitpunkt_des_fehlers);
			}
		}
		update_option('gartentechnik_com_api_timeoutfehler', $neue_fehler, true);
	}
	# VOID
	private function _schreibe_cache_fuer_standardseite(&$response) {# Referenz, um Speicher zu sparen
		update_option('gartentechnik_com_standardseite_cache', $response, false);# Kein Autoload, weil nur im Notfall noetig
	}
	# OBJ || NULL
	private function _lese_cache_fuer_standardseite() {
		return get_option('gartentechnik_com_standardseite_cache');
	}
	# OBJ
	private function _fill_response($response, $content, $use_explicit_http_response_code) {
		if(!$content) {
			$content = [];
		}
		$response->template_key(@$content['global']['template_key']);
		if($use_explicit_http_response_code) {
			$response->is_404(@$content['ist_404']);
			$response->send_explicit_http_response_code(true);
		} else {
			$response->is_404(false);
			$response->send_explicit_http_response_code(false);
		}
		$response->content_type(@$content['http_header']['content_type']);
		$response->charset(@$content['http_header']['charset']);
		$response->redirect_location(@$content['http_header']['weiterleitung']);
		if(@$content['http_header'] && array_key_exists('content_disposition', @$content['http_header'])) {
			$response->content_disposition(@$content['http_header']['content_disposition']);
		}

		$response->html_head_titel(@$content['seite']['title']);
		$response->html_head_content(@$content['seite']['head']);
		$response->html_head_allgemein_content(@$content['seite']['allgemeiner_head']);
		$response->html_head_metadescription(@$content['seite']['metadescription']);
		$response->html_content_titel(@$content['seite']['h1']);
		$response->main_content(@$content['seite']['body'], @$content['seite']['presenter_key']);
        $response->html_footer_content(@$content['seite']['footer']);
		$response->content_is_plain(@$content['seite']['is_plain']);

		$response->widget_oeffnungszeiten(@$content['widget']['oeffnungszeiten']);
		$response->widget_kontaktformular(@$content['widget']['kontaktformular']);
		$response->widget_newsletter_abonnieren(@$content['widget']['newsletter-abonnieren']);
		$response->widget_haendleradresse(@$content['widget']['haendleradresse']);
		$response->widget_angebote(@$content['widget']['angebote']);
		$response->widget_angesagte_kategorien(@$content['widget']['angesagte-kategorien']);
		$response->widget_servicegebiet(@$content['widget']['servicegebiet']);
		$response->widget_lieferanten(@$content['widget']['lieferanten']);
		$response->allgemeine_html_header(@$content['widget']['allgemeine_html_header']);
		$response->kategoriebaum(json_decode(@$content['widget']['kategoriebaum'], true));
		$response->admin_menue(json_decode(@$content['widget']['admin_menue'], true));
		$response->einstellungen(json_decode(@$content['widget']['einstellungen'], true));

		return $response;
	}
	# STRING
	public function generate_unique_post_slug($slug, $post_ID, $post_status, $post_type, $post_parent, $original_slug) {
		if($this->is_currently_retrying_slug_generation) {
			return $slug;
		}

		if($this->_ist_gtcom_pfad("/$slug/")) {
			$hit = array();
			$number = 2;
			if(preg_match('/\-(\d+)$/', $slug, $hit) === 1) {
				$number = $hit[1] + 1;
				$slug = preg_replace('/-\d+$/', '', $slug);
			}
			while($this->_ist_gtcom_pfad("/$slug-$number/")) {
				$number++;
				if($number > 10000) break;
			}

			$this->is_currently_retrying_slug_generation = true;
			while(true) {
				if($number > 15000) break;

				$new_gtcom_slug = "$slug-$number";
				$new_wp_slug = wp_unique_post_slug($new_gtcom_slug, $post_ID, $post_status, $post_type, $post_parent);
				if($new_wp_slug == "$new_gtcom_slug-2") {
					$number++;
					continue;
				}

				break;
			}
			$slug = $new_wp_slug;
			$this->is_currently_retrying_slug_generation = false;
		}
		return $slug;
	}
}
?>
