<?php
namespace GartentechnikCom;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

include_once(ABSPATH . WPINC . '/feed.php');
require_once(ABSPATH . 'wp-admin/includes/image.php');

class RSSImporter {
	private $max_items = 10000;
	private $domain = '';
	private $log_messages = [];

	# VOID
	function __construct() {
	}

	# VOID
	public function import($domain) {
		$domain = $this->_uebersetze_domain($domain);
		if($domain == 'wp.gartentechnik.com') {
			return;// diesen Blog ignorieren
		}
		$this->domain = $domain;

		$this->_log("Starte Import");

		$rss_nachrichten = $this->_hole_rss_nachrichten($domain);
		if (!count($rss_nachrichten)) {
			$this->_log("Keine RSS-Nachrichten gefunden");
			return;
		}

		$existierende_nachrichten = $this->_hole_existierende_nachrichten();
		$bereits_importierte_quell_links = $this->_gib_bereits_importierte_quell_links();
		foreach ($rss_nachrichten as $rss_nachricht) {
			$this->_log("RSS: " . $rss_nachricht['link']);
			$existierende_nachricht = $this->_gib_existierende_nachricht_zu_rss_nachricht($rss_nachricht, $existierende_nachrichten);
			if (!is_null($existierende_nachricht)) {
				$this->_log("Existiert: " . $existierende_nachricht['content_checksum'] . ' != ' . $rss_nachricht['content_checksum']);
				if (
					$existierende_nachricht['content_checksum'] != $rss_nachricht['content_checksum']
					||
					$existierende_nachricht['post_date'] != $rss_nachricht['datetime']
				) {
					$this->_log("Aktualisiere");
					$this->_aktualisiere_nachricht($existierende_nachricht, $rss_nachricht);
				}
			} else {
				$this->_log("NichtExistent: " . $rss_nachricht['link']);
				$rss_link = $rss_nachricht['link'];
				$rss_link = preg_replace('/^https?:\/\//', '', $rss_link);
				if (
					!array_key_exists("http://$rss_link", $bereits_importierte_quell_links)
					&&
					!array_key_exists("https://$rss_link", $bereits_importierte_quell_links)
				) {
					$this->_log("Erstelle");
					$this->_erstelle_neue_nachricht($rss_nachricht);
				}
			}
			$bereits_importierte_quell_links[$rss_nachricht['link']] = time();
		}
		$this->_speichere_bereits_importierte_quell_links($bereits_importierte_quell_links);
		$this->_log("Import erledigt");
		return;
	}

	# VOID
	protected function _aktualisiere_nachricht($nachricht, $rss_nachricht) {
		$tags = array();
		foreach ($rss_nachricht['kategorien'] as $kategorie) {
			array_push($tags, strtolower($kategorie));
		}
		$id = wp_update_post(array(
			'ID' => $nachricht['ID'],
			'post_title' => $rss_nachricht['titel'],
			'post_content' => $rss_nachricht['inhalt'],
			'post_date' => $rss_nachricht['datetime'],
		), true);

		if (!is_wp_error($id)) {
			wp_set_post_tags($id, implode(',', $tags), true);
			update_post_meta($id, 'gartentechnik_com_rss_import_content_checksum', $rss_nachricht['content_checksum']);

			$this->_setze_post_thumbnail($id, $rss_nachricht['inhalt']);
			$this->_log("UPDATE $id -> " . $rss_nachricht['link']);
		} else {
			$msg = "FEHLER BEI POST UPDATE: " . $id->get_error_message($id->get_error_code()) . ' -> ' . $rss_nachricht['link'];
			$this->_log($msg);
		}
		return;
	}

	# VOID
	protected function _setze_post_thumbnail($post_id, $inhalt) {
		if (preg_match('#<img [^<]*src="([^"]+)"#', $inhalt, $matches)) {
			$url = $matches[1];

			$setze_neues_bild = true;

			$bilddatei_pfadangaben = $this->_erzeuge_bilddatei_pfadangaben($url, $post_id);
			$thumbnail_id = get_post_thumbnail_id($post_id);
			if ($thumbnail_id) {
				$bild_post = get_post($thumbnail_id);
				if (!is_null($bild_post) && $bild_post->guid == $bilddatei_pfadangaben['guid']) {
					$setze_neues_bild = false;
				}
			}

			if ($setze_neues_bild) {
				$this->_log("Setze Bild $url");
				if ($this->_speichere_bilddatei_lokal($url, $bilddatei_pfadangaben)) {
					if ($this->_setze_nachrichten_bild($bilddatei_pfadangaben, $post_id)) {
						$this->_log('Bild: Erfolg');
					}
				}
			}
		}
		return;
	}

	# VOID
	protected function _erstelle_neue_nachricht($rss_nachricht) {
		$tags = array();
		foreach ($rss_nachricht['kategorien'] as $kategorie) {
			array_push($tags, strtolower($kategorie));
		}
		$id = wp_insert_post(array(
			'post_author' => 1,
			'post_date' => $rss_nachricht['datetime'],
			'post_content' => $rss_nachricht['inhalt'],
			'post_title' => $rss_nachricht['titel'],
			'post_status' => 'publish',
			'post_type' => 'post',
		), true);

		if (!is_wp_error($id)) {
			wp_set_post_tags($id, implode(',', $tags));
			update_post_meta($id, 'gartentechnik_com_rss_import_quell_link', $rss_nachricht['link']);
			update_post_meta($id, 'gartentechnik_com_rss_import_content_checksum', $rss_nachricht['content_checksum']);
			$this->_log("NEU $id -> " . $rss_nachricht['link']);

			$this->_setze_post_thumbnail($id, $rss_nachricht['inhalt']);
		} else {
			$msg = "FEHLER BEI NEUEM POST: " . $id->get_error_message($id->get_error_code()) . ' -> ' . $rss_nachricht['link'];
			$this->_log($msg);
		}
		return;
	}

	# ARRAY || NULL
	protected function _gib_existierende_nachricht_zu_rss_nachricht($rss_nachricht, $existierende_nachrichten) {
		$rss_link = $rss_nachricht['link'];
		$rss_link = preg_replace('/^https?:\/\//', '', $rss_link);
		foreach ($existierende_nachrichten as $nachricht) {
			$post_link = $nachricht['quell_link'];
			$post_link = preg_replace('/^https?:\/\//', '', $post_link);
			if ($post_link == $rss_link) {
				return $nachricht;
			}
		}
		return null;
	}

	# ARRAY
	protected function _hole_existierende_nachrichten() {
		$args = array(
			'posts_per_page' => -1,
			'offset' => 0,
			'orderby' => 'date',
			'order' => 'DESC',
			'post_type' => 'post',
			'post_status' => 'any',
			'suppress_filters' => true,
		);
		$posts_array = get_posts($args);
		$posts = array();
		foreach ($posts_array as $post) {
			$quell_link = $this->_gib_post_quell_link($post->ID);
			array_push($posts, array(
				'ID' => $post->ID,
				'quell_link' => $quell_link,
				'content_checksum' => $this->_gib_post_content_checksum($post->ID, $quell_link),
				'titel' => apply_filters('the_title', $post->post_title),
				'status' => $post->post_status,
				'post_date' => $post->post_date,
			));
		}
		return $posts;
	}

	# STRING
	protected function _erzeuge_bilddatei_pfadangaben($url, $post_id) {
		$wp_upload_dir = wp_upload_dir();
		$dateiname = 'gartentechnik_com_rss_nachricht_' . $post_id . '_' . basename($url);
		return array(
			'dateiname' => $dateiname,
			'pfad' => $wp_upload_dir['path'],
			'guid' => $wp_upload_dir['url'] . "/$dateiname",
		);
	}

	# BOOLEAN
	protected function _speichere_bilddatei_lokal($url, $bilddatei_pfadangaben) {
		$response = wp_remote_get($url, array());
		if ($response['response']['code'] != 200) {
			return null;
		}

		$bilddaten = $response['body'];
		if (!$bilddaten) {
			return null;
		}

		@mkdir($bilddatei_pfadangaben['pfad'], null, true);
		if (!file_put_contents($bilddatei_pfadangaben['pfad'] . '/' . $bilddatei_pfadangaben['dateiname'], $bilddaten)) {
			return false;
		}
		return true;
	}

	# BOOLEAN
	protected function _setze_nachrichten_bild($bilddatei_pfadangaben, $post_id) {
		$mime_type = wp_check_filetype($bilddatei_pfadangaben['dateiname'], null);
		$attachment = array(
			'guid' => $bilddatei_pfadangaben['guid'],
			'post_mime_type' => $mime_type['type'],
			'post_title' => preg_replace('/\.[^.]+$/', '', $bilddatei_pfadangaben['dateiname']),
			'post_content' => '',
			'post_status' => 'inherit'
		);
		$dateipfad = $bilddatei_pfadangaben['pfad'] . '/' . $bilddatei_pfadangaben['dateiname'];
		$attach_id = wp_insert_attachment($attachment, $dateipfad, $post_id);
		if (!$attach_id) {
			return false;
		}

		$attach_data = wp_generate_attachment_metadata($attach_id, $dateipfad);
		if (!wp_update_attachment_metadata($attach_id, $attach_data)) {
			return false;
		}
		$meta_id = set_post_thumbnail($post_id, $attach_id);
		if (!$meta_id) {
			return false;
		}
		return true;
	}

	# STRING
	protected function _gib_post_quell_link($id) {
		return get_post_meta($id, 'gartentechnik_com_rss_import_quell_link', true);
	}

	# STRING
	protected function _gib_post_content_checksum($id, $quell_link) {
		if (!$quell_link) return '';

		return get_post_meta($id, 'gartentechnik_com_rss_import_content_checksum', true);
	}

	# VOID
	protected function _speichere_bereits_importierte_quell_links($bereits_importierte_quell_links) {
		update_option('gartentechnik_com_rss_import_bereits_importierte_quell_links', $bereits_importierte_quell_links);
	}

	# ARRAY
	protected function _gib_bereits_importierte_quell_links() {
		$bereits_importierte_quell_links = get_option('gartentechnik_com_rss_import_bereits_importierte_quell_links');
		if (!is_array($bereits_importierte_quell_links)) {
			$bereits_importierte_quell_links = array();
		}
		return $bereits_importierte_quell_links;
	}

	# STRING
	protected function _uebersetze_domain($domain) {
		$domain = preg_replace('/^(?:preview)\.(.+)\.gartentechnik\.com/', '$1', $domain);
		$domain = preg_replace('/\.vm$/', '', $domain);
		return $domain;
	}

	# ARRAY
	protected function _hole_rss_nachrichten($domain) {
		$eintraege = array();
		$url = 'https://www.gartentechnik.com/abonnierte_rss_nachricht?motorist=' . $domain . '&x=1';

		$timeout = function () {
			return 60;
		};
		add_filter('wp_feed_cache_transient_lifetime', $timeout);
		$rss = fetch_feed($url);
		remove_filter('wp_feed_cache_transient_lifetime', $timeout);

		$item_count = 0;
		if (!is_wp_error($rss)) {
			$item_count = $rss->get_item_quantity($this->max_items);
		} else {
			$msg = "Fehler bei $url: " . $rss->get_error_message($rss->get_error_code());
			$this->_log($msg);
			return [];
		}

		if ($item_count == 0) {
			$this->_log("Keine Items");
			return $eintraege;
		}

		$rss_items = $rss->get_items(0, $item_count);
		if (!$rss_items) {
			$this->_log("Keine Items ausgelesen");
			return $eintraege;
		}

		$i = 0;
		foreach ($rss_items as $item) {
			$i++;
			$daten = $this->_konvertiere_rss_zu_nachricht($item);
			if (!$daten) {
				continue;
			}
			array_push($eintraege, $daten);
		}
		return $eintraege;
	}

	# VOID
	protected function _log($msg) {
		$log_msg = "RSS-IMPORT " . $this->domain . ": " . $msg;
		array_push($this->log_messages, $log_msg);
		gartentechnik_com_debug($log_msg);
		return;
	}

	# STRING
	public function get_log_messages() {
		return implode("\n", $this->log_messages);
	}

	# ARRAY || NULL
	protected function _konvertiere_rss_zu_nachricht($item) {
		$category_list = $item->get_categories();
		$categories = array();
		if ($category_list) {
			foreach ($category_list as $cat) {
				$label = $cat->get_label();
				if ($label) {
					array_push($categories, $label);
				}
			}
		}
		$link = $item->get_permalink();
		$link = preg_replace('/\.vm\//', '/', $link);
		$i = array(
			'titel' => $item->get_title(),
			'link' => $link,
			'inhalt' => $item->get_content(),
			'kategorien' => $categories,
			'autor' => ($item->get_author() ? $item->get_author()->get_name() : ''),
			'timestamp' => ($item->get_date('U') ? $item->get_date('U') : 0),
			'datetime' => ($item->get_date('Y-m-d h:i:s') ? $item->get_date('Y-m-d h:i:s') : null),
			'content_checksum' => '',
		);
		if (
			!$i['titel']
			|| !$i['link']
			|| !$i['inhalt']
			|| !$i['timestamp']
		) {
			return null;
		}

		$i['content_checksum'] = md5(
			implode(
				array(
					"\n\r|wcqnrz^8927ey\twethcn13",
					$i['titel'],
					$i['inhalt'],
					implode(',', $i['kategorien'])
				)
			)
		);
		return $i;
	}
}

?>
