<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/*
Version:     2.5.3
Plugin Name: Gartentechnik.com
Plugin URI:  https://wordpress.gartentechnik.com/
Description: Gartentechnik.com ist ein kostenfreies, open source WordPress-Plugin zur Benutzung der kostenpflichtigen API des gleichnamigen Service für Motorgerätefachhändler und Motoristen.
Text Domain: gartentechnik-com
Author:      Gartentechnik.communications GmbH
Author URI:  https://www.gartentechnik.com/
License:     GPL3
License URI: https://www.gnu.org/licenses/gpl-3.0.html
*/

$gartentechnik_com_domainname = 'gartentechnik-com';
#$gartentechnik_debug_request_id = rand();
$gartentechnik_debug_start_zeit = time() + sprintf('%.4f', microtime());
$gartentechnik_com_pluginpfad = __FILE__;
$gartentechnik_com_api_version = '2';
$gartentechnik_com_use_header_debug = false;
$gartentechnik_com_use_header_debug_config = [];
$gartentechnik_com_use_header_debug_counter = 0;
$gartentechnik_com_special_post_id = -918273645;

# VOID
function gartentechnik_com_debug($msg) {
	#global $gartentechnik_debug_request_id;
	global $gartentechnik_debug_start_zeit;
	global $gartentechnik_com_use_header_debug;
	global $gartentechnik_com_use_header_debug_config;
	global $gartentechnik_com_use_header_debug_counter;

	if($gartentechnik_com_use_header_debug) {
		$gartentechnik_com_use_header_debug_counter++;
		$laufzeit = (time() + sprintf('%.4f', microtime()) - $gartentechnik_debug_start_zeit);
		$start = $gartentechnik_com_use_header_debug_config['offset'];
		$ende = $gartentechnik_com_use_header_debug_config['offset'] + $gartentechnik_com_use_header_debug_config['limit'];
		if(
			$gartentechnik_com_use_header_debug_counter >= $start
			&& $gartentechnik_com_use_header_debug_counter <= $ende
		) {
			if($gartentechnik_com_use_header_debug_counter == 1) {
				$fh = fopen(__FILE__, "r");
				$file_content = '';
				if($fh) {
					$file_content = fread($fh, filesize(__FILE__));
					fclose($fh);
				}
				$v = '-';
				if(preg_match('/Version: *([\d\.]+)/', $file_content, $hits)) {
					$v = $hits[1];
				}
				header(
					"X-GT00: "
					. substr("$v", 0, $gartentechnik_com_use_header_debug_config['str'])
				);
			}
			header(
				"X-GT" . sprintf('%02d', $gartentechnik_com_use_header_debug_counter) . ": "
				. substr("$msg ($laufzeit)", 0, $gartentechnik_com_use_header_debug_config['str'])
			);
		}
	}
	#trigger_error(
	#	"GTCOM[$gartentechnik_debug_request_id]:$msg",
	#	E_USER_NOTICE
	#);
}

require_once(__DIR__ . '/_gt4.php');

include_once(__DIR__ . '/_rss_import.php');

add_filter('init', function() use($gartentechnik_com_domainname, $gartentechnik_com_api_version) {
	global $gartentechnik_com_use_header_debug;
	global $gartentechnik_com_use_header_debug_config;
	if(preg_match('/\?gartentechnik_com_use_header_debug=offset(\d+)limit(\d+)str(\d+)/', getenv('REQUEST_URI'), $hits)) {
		$gartentechnik_com_use_header_debug = true;
		$gartentechnik_com_use_header_debug_config = [
			'offset' => $hits[1],
			'limit' => $hits[2],
			'str' => $hits[3],
		];
		add_filter('wp_redirect', function($url) {
			$stack = debug_backtrace();
			foreach(array_splice($stack, 0, 15) as $caller) {
				gartentechnik_com_debug("R($url): " . $caller['file'] . '(' . $caller['line'] . ')');
			}
		});
	}
	gartentechnik_com_debug('init_start! req:' . @$_SERVER['REQUEST_URI'] . ' referrer:' . @$_SERVER['HTTP_REFERER']);

	if(
		array_key_exists('GARTENTECHNIK_COM_IS_DOCKER_CONTEXT', $_SERVER)
		&& $_SERVER['GARTENTECHNIK_COM_IS_DOCKER_CONTEXT'] == '1'
		&& preg_match('/^\/fotos\/sites\//', getenv('REQUEST_URI'))
	) {
		gartentechnik_com_debug("abort; internal image content");
		return;
	}

	$url_path_starts_with = function($haystack, $needle) {
		return (substr($haystack, 0, strlen($needle)) === $needle);
	};
	if(
		$url_path_starts_with(getenv('REQUEST_URI'), home_url('/wp-content', 'relative'))
		||
		$url_path_starts_with(getenv('REQUEST_URI'), home_url('/wp-includes', 'relative'))
	) {
		gartentechnik_com_debug("abort; internal content");
		return;
	}

	$domain = $_SERVER['HTTP_HOST'];
	$url_schema = $_SERVER['HTTPS'] == 'on' ? 'https' : 'http';
	if(preg_match('/^(https?):\/\/([^\/]+)/', home_url('/'), $wp_url_hits)) {
		$domain = $wp_url_hits[2];
		$url_schema = $wp_url_hits[1];
	}

	$current_path = $_SERVER['REQUEST_URI'];
	$current_path_prefix = preg_replace('/\/$/', '', home_url('/', 'relative'));
	if(strlen($current_path_prefix) > 0) {
		$current_path = substr($current_path, strlen($current_path_prefix));
	}
	gartentechnik_com_debug("HOST: '$url_schema' + '$domain' + '$current_path_prefix' + '$current_path'");

	load_plugin_textdomain($gartentechnik_com_domainname, false, dirname( plugin_basename(__FILE__) ) . '/languages/' );
	gartentechnik_com_debug('Load language: ' . __('Language: English', $gartentechnik_com_domainname));

	require_once(ABSPATH . 'wp-admin/includes/plugin.php');
	require_once(__DIR__ . '/classes/Response.php');
	require_once(__DIR__ . '/classes/MotoristPlugin.php');
	require_once(__DIR__ . '/classes/Register.php');
	require_once(__DIR__ . '/classes/EinstellungsRegister.php');
	require_once(__DIR__ . '/classes/Linkerzeuger.php');
	require_once(__DIR__ . '/classes/WP/Menue.php');

	\GartentechnikCom\EinstellungsRegister::init();

	if(wp_doing_ajax()) {
		$widget_id = @$_GET['widget-id'] ? sanitize_text_field(@$_GET['widget-id']) : sanitize_text_field(@$_POST['widget-id']);
		if(!$widget_id) $widget_id = '';
		$action = @$_GET['action'] ? sanitize_text_field(@$_GET['action']) : sanitize_text_field(@$_POST['action']);
		if(!$action) $action = '';
		if(
			array_key_exists('s', $_POST)
			&& strlen(sanitize_text_field($_POST['s'])) > 0
			&& $_SERVER['REQUEST_METHOD'] == 'POST'
			&& preg_match('/ajax_search$/', $action)
		) {
			\GartentechnikCom\Register::ist_such_autovervollstaendigung(true);
		} else if(
			preg_match('/^gartentechnik_com/', $action)
			|| preg_match('/^gartentechnik\-com/', $widget_id)
		) {
			# Weiter normal behandeln
		} else {
			return;
		}
	}

	$ist_superadmin_kontext = (preg_match('/^wp\.gartentechnik\.com(?:\.vm|\.test\.server\.gartentechnik\.com|)$/', $domain) ? true : false);
	if(preg_match('/^preview\.(.*)\.gartentechnik\.com(.*)$/', $domain, $treffer)) {
		$domain = $treffer[1] . ($treffer[2] ? $treffer[2] : '');
	}

	$current_path = preg_replace('/\?.*$/', '', $current_path);
	$cgi_parameter = $_SERVER['REQUEST_METHOD'] == 'GET' ? $_GET : $_POST;
	$cgi_parameter = stripslashes_deep($cgi_parameter);

	$weiterleitungs_pfad = \GartentechnikCom\EinstellungsRegister::gib_pfad_weiterleitung($current_path);
	if($weiterleitungs_pfad) {
		header("HTTP/1.1 301 MovedPermanently");
		header("Location: $weiterleitungs_pfad");
		gartentechnik_com_debug('weiterleitung');
		exit(0);
	}

	gartentechnik_com_debug('rechtepruefung');
	$ist_admin_kontext = false;
	$rolle_des_aktuellen_users = '';
	if(is_user_logged_in()) {
		if(
			in_array('hersteller', ( array ) wp_get_current_user()->roles)
			&&
			current_user_can('edit_pages')
		) {
			$rolle_des_aktuellen_users = 'hersteller';
		} elseif(current_user_can('manage_network')) {
			$rolle_des_aktuellen_users = 'admin';
			if($ist_superadmin_kontext) {
				$rolle_des_aktuellen_users = 'superadmin';
			}
		} elseif(current_user_can('manage_options')) {
			$rolle_des_aktuellen_users = 'admin';
		} elseif(
			current_user_can('edit_pages')
			|| (
				in_array('gartentechnik_com_only', ( array ) wp_get_current_user()->roles)
				&&
				current_user_can('edit_posts')
			)
		) {
			$rolle_des_aktuellen_users = 'editor';
		}
		if($rolle_des_aktuellen_users) {
			$ist_admin_kontext = true;
		}
	}

	$app = new \GartentechnikCom\MotoristPlugin();
	$app->init($url_schema, $domain, $current_path_prefix);
	\GartentechnikCom\Register::init(
		$domain,
		$current_path,
		$app,
		get_plugin_data( __FILE__)['Version'],
		$gartentechnik_com_api_version
	);

	if(
		$ist_admin_kontext
		&& $_SERVER['REQUEST_METHOD'] == 'POST'
		&& array_key_exists('gartentechnik_com_action', $cgi_parameter)
		&& $cgi_parameter['gartentechnik_com_action'] == 'generiere_testaccount_api_key'
	) {
		gartentechnik_com_debug('gtcom_test_api_key_start');
		if($app->starte_api_key_testzeitraum(
			$domain,
			$ist_admin_kontext,
			$ist_superadmin_kontext,
			$rolle_des_aktuellen_users
		)) {
			gartentechnik_com_debug('gtcom_test_api_key_success');
			header("HTTP/1.1 302 MovedTemporary");
			header("Location: " . $_SERVER['REQUEST_URI']);
			gartentechnik_com_debug('gtcom_test_api_key_end');
			exit(0);
		}
	}

	add_filter('robots_txt', function($content) use($domain) {
		gartentechnik_com_debug('robots_start');
		# Der Google-Bot folgt den Trackingtexten, die die API-URL enthalten und wie ein Linkpfad aussehen, aber keiner sind :(
		$verbotene_pfade = "\nDisallow: /v" . \GartentechnikCom\Register::gib_api_version() . '/api';

		$slug = \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('agb');
		if(!$slug) $slug = 'agb';
		if(\GartentechnikCom\EinstellungsRegister::ist_gtcom_slug_aktiv('agb')) {
			$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$slug/", 'relative');
		}

		$slug = \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('datenschutz');
		if(!$slug) $slug = 'datenschutz';
		if(\GartentechnikCom\EinstellungsRegister::ist_gtcom_slug_aktiv('datenschutz')) {
			$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$slug/", 'relative');
		}

		# Gedoppelt in der MotoristPlugin.pm
		$kaufen_slug = \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('kaufen');
		$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$kaufen_slug*/marke-+", 'relative');
		$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$kaufen_slug*/zustand-+", 'relative');
		$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$kaufen_slug*/preis-*EUR/$", 'relative');
		$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$kaufen_slug*/sortierung-preis-aufsteigend/$", 'relative');
		$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$kaufen_slug*/sortierung-preis-absteigend/$", 'relative');

		gartentechnik_com_debug('robots_end');
		return $content
			. $verbotene_pfade
			. "\nCrawl-delay: 1"
			. "\nSitemap: " . home_url("/$kaufen_slug/sitemap.xml") . "\n";
	}, 1000, 1);
	add_filter('wp_unique_post_slug', function($slug, $post_ID, $post_status, $post_type, $post_parent, $original_slug) use ($app) {
		gartentechnik_com_debug('suche_eindeutigen_slug');
		return $app->generate_unique_post_slug($slug, $post_ID, $post_status, $post_type, $post_parent, $original_slug);
	}, 10, 6);

	if(!is_admin() && array_key_exists('s', $cgi_parameter)) {
		\GartentechnikCom\Register::ist_eine_suche(true);
		\GartentechnikCom\Register::suchquery($cgi_parameter['s']);
	}
	gartentechnik_com_debug('request_pruefen_start');

	// Auch im globalen Account die StandardSitemap zulassen
	//	if(
	//		array_key_exists('GARTENTECHNIK_COM_IS_DOCKER_CONTEXT', $_SERVER)
	//		&& $_SERVER['GARTENTECHNIK_COM_IS_DOCKER_CONTEXT'] == '1'
	//		&& $current_path == "/sitemap.xml"
	//	) {
	//		wp_redirect(
	//			home_url('/' . \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('kaufen') . '/sitemap.xml', 'relative')
	//		);
	//		gartentechnik_com_debug('sitemap wird gerufen');
	//		exit(0);
	//	}

	gartentechnik_com_debug("api-version=" . \GartentechnikCom\Register::gib_api_version());
	if(
		($ist_admin_kontext || $ist_superadmin_kontext)
		&& array_key_exists('action', $cgi_parameter)
		&& $cgi_parameter['action'] == 'gartentechnik_com_admin_api_anfrage'
	) {
		# DoNothing: da wird nur die API gefragt, sonst nix
		$response = null;
	} elseif($app->ist_gtcom_request($_SERVER['REQUEST_METHOD'], $current_path)) {
		gartentechnik_com_debug('gtcom_request_start');
		$response = $app->execute_gtcom_request(
			$_SERVER['REQUEST_METHOD'],
			$current_path,
			$cgi_parameter,
			$ist_admin_kontext,
			$ist_superadmin_kontext,
			$rolle_des_aktuellen_users
		);
		gartentechnik_com_debug('gtcom_request_ende');
		\GartentechnikCom\Register::zeige_gtcom(true);
		remove_filter('the_content', 'wpautop');# Sonst fuegt WP um viele Elemente im Content ein P-Tag
		if($response->explicit_response_code()) {
			\GartentechnikCom\Register::zeige_gtcom(false);
			gartentechnik_com_debug($response->explicit_response_code());
			header("HTTP/1.1 " . $response->explicit_response_code());
			echo "<html><body><h1>" . $response->explicit_response_code() . "</h1></body></html>";
			exit(0);
		} elseif($response->is_404() || $response->redirect_location()) {
			\GartentechnikCom\Register::zeige_gtcom(false);
			gartentechnik_com_debug('404');
			add_filter('request', function($request) {
				return array('error' => '404');
			}, 20, 1);
		} elseif($response->content_is_plain()) {
			gartentechnik_com_debug('plaintext');
			header("HTTP/1.1 200 OK");
			header('Content-Type: ' . $response->content_type() . "; charset=" . $response->charset());
			echo $response->main_content();
			exit(0);
		}
	} else {
		gartentechnik_com_debug('wp_request_start');
		$response = $app->execute_other(
			$cgi_parameter,
			$ist_admin_kontext,
			$ist_superadmin_kontext,
			$rolle_des_aktuellen_users
		);
		gartentechnik_com_debug('wp_request_end');
		$ueberschreibe_standardsuche = apply_filters('gartentechnik_com_ueberschreibe_standardsuche', true);
		if($ueberschreibe_standardsuche && \GartentechnikCom\Register::ist_eine_suche()) {
			$response->redirect_location(
				home_url('/' . \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('kaufen') . '/', 'relative')
				. '?s=' . urlencode(\GartentechnikCom\Register::suchquery())
			);
			gartentechnik_com_debug('suche_ausfuehren');
			add_filter('request', function($request) {
				return array('error' => '404');
			}, 20, 1);
		}
	}

	require_once(__DIR__ . '/_allgemeine_funktionen.php');
	require_once(__DIR__ . '/_autovervollstaendigung_suchfeld.php');
	require_once(__DIR__ . '/_produktsuche_fuer_produktwidget.php');
	require_once(__DIR__ . '/_produktwidget_datenabruf.php');
	require_once(__DIR__ . '/_zeige_gt_inhalte_inline.php');
	if(!$ist_superadmin_kontext && $rolle_des_aktuellen_users != 'hersteller') {
		require_once(__DIR__ . '/_menu_post_type.php');
	}
	require_once(__DIR__ . '/_gtcom_adminbereich.php');

	if(is_user_logged_in()) {
		if(!extension_loaded('xml')) {
			gartentechnik_com_wp_fehlermeldung('Das Gartentechnik.com-Plugin benötigt das PHP-Modul "xml" um abonnierte RSS-Nachrichten importieren und Angebots-Widgets anzeigen zu können. Bitte installieren Sie dieses Modul (bei Debian z.B. php-xml).');
		}
		if(!extension_loaded('pcre')) {
			gartentechnik_com_wp_fehlermeldung('Das Gartentechnik.com-Plugin benötigt das PHP-Modul "pcre" um abonnierte RSS-Nachrichten importieren zu können. Bitte installieren Sie dieses Modul.');
		}
		if(!extension_loaded('iconv')) {
			gartentechnik_com_wp_fehlermeldung('Das Gartentechnik.com-Plugin benötigt das PHP-Modul "iconv" um abonnierte RSS-Nachrichten importieren zu können. Bitte installieren Sie dieses Modul.');
		}
	}

	add_action('widgets_init', function() {
		gartentechnik_com_debug('widget_init_start');
		$registered_widgets = [];
		foreach([
			'Oeffnungszeiten',
			'Kontaktformular',
			'NewsletterAbonnieren',
			'Haendleradresse',
			'Angebote',
			'AngesagteKategorien',
			'Servicegebiet',
			'Lieferanten',
			'Produkt'
		] as $name) {
			require_once(__DIR__ . '/classes/Widget/' . $name . '.php');
			$classname = '\GartentechnikCom\Widget\\' . $name;
			register_widget($classname);
			// AMR-plugin wird damit ersetzt
			if(!function_exists('amr_do_widget')) {
				$registered_widgets[$name] = new $classname();
			}
		}
		if(!function_exists('amr_do_widget')) {
			$number = 0;
			add_shortcode('do_widget', function ($atts) use ($registered_widgets, &$number) {
				if (is_array($atts) && count($atts) == 1) {
					foreach($registered_widgets as $key => $obj) {
						$optionen = [];
						if($key == 'Angebote') {
							$optionen['zeilen_angebote'] = 1;
							$optionen['angebote_auffuellen'] = 3;
						}
						if(array_key_exists('0', $atts) && $atts[0] == 'G ' . $key) {
							$number++;
							return '<section id="gartentechnik-com-shortcode-' . $number . '" class="widget clearfix widget_gartentechnik-com-' . strtolower($key) . ' gartentechnik-com-zeige-in-druckansicht">'
							. $obj->get_widget_content($optionen)
                            . '</section>';
						}
					}
				}
				return "[INVALID SHORTCODE 'do_widget']";
			});
		}
		gartentechnik_com_debug('widget_init_end');
	}, 20, 0);

	add_action('wp_head', function() use ($response, $current_path) {
		gartentechnik_com_debug('wp_head_start');
		if($response) {
			$zeige_gtcom_content = GartentechnikCom\Register::zeige_gtcom();
			require_once(__DIR__ . '/templates/allgemeine_html_header.html');
			echo $response->html_head_allgemein_content();
			if($zeige_gtcom_content) {
				echo $response->html_head_content();
			}
		}
		gartentechnik_com_debug('wp_head_stop');
	}, 20, 0);

	add_action('admin_head', function() use ($response, $current_path) {
		gartentechnik_com_debug('admin_head_start');
		if($response) {
			echo $response->html_head_allgemein_content();
			echo $response->html_head_content();
		}
		gartentechnik_com_debug('admin_head_start');
	}, 20, 0);

	add_action('wp_footer', function() use ($response) {
		gartentechnik_com_debug('wp_footer_start');
		if($response) {
	        echo $response->html_footer_content();
			\GartentechnikCom\EinstellungsRegister::finish($response);
		}
		gartentechnik_com_debug('wp_foter_end');
	}, 20, 0);
	add_action('admin_footer', function() use ($response) {
		gartentechnik_com_debug('admin_footer_start');
		if($response) {
			echo $response->html_footer_content();
			\GartentechnikCom\EinstellungsRegister::finish($response);
		}
		gartentechnik_com_debug('admin_footer_end');
	}, 20, 0);
	add_role(
		'hersteller',
		__('Hersteller'),
		get_role('editor')->capabilities
	);
	add_role(
		'gartentechnik_com_only', __('nur Gartentechnik'),
		array(
			'read' => true, // so users can read
			'edit_posts' => true,
		)
	);
	gartentechnik_com_debug('init_end');
}, 0, 0);

?>
